/*
 * P2HashSet.java
 *
 * Created on March 15, 2005, 1:40 PM
 */

package jxl.peermi;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;


/**
 * A HashSet containing only PeerRemote objects. The purpose of this class is to 
 * insure that any remote invocation made on the contained objects will be made on 
 * alive (that is, running) remote hosts. To do this, PeerHashSet calls 
 * {@link jxl.peermi.PeerRemote#checkAlive()} on ever object added to the collection, 
 * and on every object before it is returned from the iterator.
 * @author Alex Lynch
 */
public class PeerHashSet<E extends PeerRemote> extends HashSet<E>{
    
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** Creates a new instance of P2HashSet */
    public PeerHashSet() {
    }
    public boolean add(E e){
        if(PeerRemoteObject.isAlive(e)){
            return super.add(e);
        }else{
            return false;
        }
    }
    
    public boolean addAll(Collection<? extends E> c){
        boolean modified = false;
        for(E e: c){
            modified |= add(e);
        }
        return modified;
    }
    
    public Iterator<E> iterator(){
        final Iterator<E> i  = super.iterator();
        return new Iterator<E>(){
            E next = null;
            public boolean hasNext(){
                if(i.hasNext()){
                    next = i.next();
                    if(PeerRemoteObject.isAlive(next)){
                        return true;
                    }else{
                        i.remove();
                        return hasNext();
                    }
                }else{
                    return false;
                }
            }
            public E next(){
                return next;
            }
            public void remove(){
                i.remove();
            }
        };
    }
}
