/*
 * PeerRemoteObject.java
 *
 * Created on March 13, 2005, 8:46 AM
 */

package jxl.peermi;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * PeerRemoteObject must be extended to create a custom Peermi remote object.
 * The subclass must implement at lease one PeerRemote interface.
 * 
 * @author Alex Lynch
 */
public abstract class PeerRemoteObject extends UnicastRemoteObject implements PeerRemote {
	/**
     * Calls <CODE>r.checkAlive()</CODE>. If a RemoteException occurs, return
     * false, else true. 
     * 
     * @param r
     *            The PeerRemote object to check.
     */
	public static final boolean isAlive(PeerRemote r) {
		try {
			r.checkAlive();
			return true;
		} catch (RemoteException re) {
			return false;
		}
		
	}
	
	
//
//	@Override
//	public static PeerRemote toStub(java.rmi.Remote r) throws NoSuchObjectException {
//		
//		if (PeerRemote.class.isInstance(r)) {
//			PeerRemote p = (PeerRemote) UnicastRemoteObject.toStub(r);
//			try {
//				// this is absolutly neccessary
//				p.checkAlive();// dont ask me why
//				// think of it as initilization.
//			} catch (RemoteException re) {
//				re.printStackTrace();// should never happen
//			}
//			return p;
//		} else {
//			throw new IllegalArgumentException("Argument must be of type " + PeerRemote.class.getName());
//		}
//	}

	/**
     * Just creates a PeerRemoteObject ready to use Peermi.
     */
	public PeerRemoteObject() throws RemoteException {
		this(0);
	}

	/**
     * Creates a PeerRemoteObject. If registerByClass is true then, bind this
     * object in the local PeerRegistry like so <CODE>PeerRegistry.getLocalInstance().bind(this,
     * this.getClass().getName());</CODE>
     * 
     * @param registerByClass
     *            if true register by class name
     * @throws java.rmi.RemoteException
     */
	public PeerRemoteObject(boolean registerByClass) throws RemoteException {
		this(0);
		if (registerByClass)
			PeerRegistry.getLocalInstance().bind(this, this.getClass().getName());
	}

	/**
     * Creates a new instance of PeerRemoteObject on a specified 'port.' This
     * option is included to support the 'normal' client-server view of rmi.
     */
	public PeerRemoteObject(int port) throws RemoteException {
		super(port);
	}

	/**
     * Always returns true. This method is called by
     * {@link jxl.peermi.PeerRemoteObject#isAlive(PeerRemote)}
     */
	public final void checkAlive() throws RemoteException {
		// System.out.println("Somebody loves me");
	}

}
